SystemOrganization addCategory: #TestBrowser!
SystemOrganization addCategory: #'TestBrowser-Tests'!
SystemOrganization addCategory: #'TestBrowser-Tests-Utilities'!

!SystemWindow methodsFor: '*TestBrowser' stamp: 'tbn 11/5/2004 10:47'!
removePanes
	"remove all current panes"

	paneMorphs do: [ :pn |
		pn delete.
		updatablePanes := updatablePanes copyWithout: pn ].
	paneMorphs := Array new.! !

Browser subclass: #TestBrowser
	instanceVariableNames: 'details passFail tests passFailText detailsText selectedFailureTest selectedErrorTest pattern systemCategoryList topWindowMorph testPaneVisible filterMode testHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TestBrowser'!

!TestBrowser commentStamp: 'tbn 11/5/2004 09:35' prior: 0!
TestBrowser represents an enhanced TestRunner for Squeak.

To open:
    TestBrowser open. (or from 'open...' menu)

It supports:

    * Filtering test cases by class category
    * You can run and edit test cases/codes in one window 

For more details:

http://minnow.cc.gatech.edu/squeak/3891
!

!TestBrowser class methodsFor: 'instance creation' stamp: 'tbn 11/5/2004 08:54'!
fullOnCategory: aClassCategory 
	"Open a new instance of the receiver on the given class category"
 
	| browser |
	browser := self new.
	browser setCategory: aClassCategory.
	self 
		openBrowserView: (browser openEditString: nil) 
		label: 'Test Browser'! !

!TestBrowser class methodsFor: 'instance creation' stamp: 'tbn 11/5/2004 08:54'!
fullOnClass: aClass 
	"Open a new instance of the receiver on the given class"
	
	| browser |
	browser := self new.
	browser setClass: aClass selector: nil.
	self
		openBrowserView: (browser openEditString: nil)
		label: 'Test Browser'! !

!TestBrowser class methodsFor: 'instance creation' stamp: 'tbn 11/5/2004 08:54'!
fullOnClass: aClass selector: aSelector 
	"Open a new instance of the receiver on the given class and selector"
	
	| browser |
	browser := self new.
	browser setClass: aClass selector: aSelector.
	self
		openBrowserView: (browser openEditString: nil)
		label: browser labelString! !

!TestBrowser class methodsFor: 'class initialization' stamp: 'tbn 11/5/2004 13:00'!
initialize
	"Initializes the receiver"

	self registerInWorldMenu.
	self registerPreferences! !

!TestBrowser class methodsFor: 'instance creation' stamp: 'tbn 11/5/2004 08:54'!
open
	"Open a new instance of the receiver"
	
	self openBrowser! !

!TestBrowser class methodsFor: 'instance creation' stamp: 'tbn 11/5/2004 08:55'!
openBrowser
	"Open a new instance of the receiver"
	
	| browser |
	browser := super new initialize.
	self
		openBrowserView: (browser openEditString: nil)
		label: browser defaultBrowserTitle! !

!TestBrowser class methodsFor: 'registration' stamp: 'tbn 11/5/2004 09:51'!
registerInWorldMenu
	"Register the receiver in the squeak world menu"
	
	 (TheWorldMenu respondsTo: #registerOpenCommand:) ifTrue: [ 
		TheWorldMenu registerOpenCommand: { 'TestBrowser' . { TestBrowser . #open }. '"TestBrowser", a Combined Browser for SUnit' }]! !

!TestBrowser class methodsFor: 'registration' stamp: 'minami 11/7/2004 18:49'!
registerPreferences
	"Registers a preference to run abstract test classes"

	Preferences
		addPreference: #testBrowserRunsAbstractClasses
		categories: #(#TestBrowser )
		default: false
		balloonHelp: 'If true, the test browser runs abstract classes'! !

!TestBrowser class methodsFor: 'class initialization' stamp: 'tbn 11/5/2004 10:01'!
removeFromSystem
	"Removes the receiver from the system"
	
	self unload.
	super removeFromSystem! !

!TestBrowser class methodsFor: 'class initialization' stamp: 'tbn 11/5/2004 13:01'!
unload
	"This method is called when the receiver is unloaded from the system"
	
	self unregisterFromWorldMenu.
	self unregisterPreferences! !

!TestBrowser class methodsFor: 'registration' stamp: 'tbn 11/5/2004 10:00'!
unregisterFromWorldMenu
	"Unregisters the receiver from the squeak world menu"
	
	(TheWorldMenu respondsTo: #registerOpenCommand:) ifTrue: 
		[TheWorldMenu unregisterOpenCommand: 'TestBrowser'].! !

!TestBrowser class methodsFor: 'registration' stamp: 'tbn 11/5/2004 13:00'!
unregisterPreferences
	"Unregisters the receicers preferences"

	Preferences removePreference: #testBrowserRunsAbstractClasses! !

!TestBrowser class methodsFor: 'version' stamp: 'minami 11/6/2004 22:34'!
version
	"Returns the receivers version string"

	| label mcVersion |
	label := 'TestBrowser' copy.
	mcVersion := self versionFromMonticello.
	mcVersion isNil ifTrue: [^ label].
	^ label , '-' , mcVersion! !

!TestBrowser class methodsFor: 'version' stamp: 'tbn 11/5/2004 10:30'!
versionFromMonticello
	"Returns the receivers version string extracted from the monticello package.
	 Returns nil if Monticello is not present.
	"

	| wc versionName version |
	self environment at: #MCWorkingCopy ifPresent: [:cls |
		wc := cls allManagers 
				detect: [:each | each packageName = self category asString]
				ifNone: [].
		wc isNil ifTrue: [^nil].
		versionName := wc ancestry ancestors first name.
		version := versionName 
				copyFrom: (versionName indexOf: $.) + 1
				to: versionName size].
	^version			! !

!TestBrowser methodsFor: 'processing' stamp: 'minami 11/7/2004 19:06'!
abstractOrNotATest
	self runWindow.
	self displayPassFail: 'Abstract or not a test '! !

!TestBrowser methodsFor: 'interface opening-morphic' stamp: 'tbn 11/5/2004 10:42'!
addLowerPanesTo: window at: nominalFractions

	| verticalOffset row innerFractions |

	row := AlignmentMorph newColumn
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		layoutInset: 0;
		borderWidth: 1;
		borderColor: Color black;
		layoutPolicy: ProportionalLayout new.

	verticalOffset := 0.
	innerFractions := 0@0 corner: 1@0.
	verticalOffset := self addOptionalAnnotationsTo: row at: innerFractions plus: verticalOffset.
	verticalOffset := self addOptionalButtonsTo: row  at: innerFractions plus: verticalOffset.

	row 
		addMorph: (self buildBrowserCodeMorph borderWidth: 0)
		fullFrame: (
			LayoutFrame 
				fractions: (innerFractions withBottom: 1) 
				offsets: (0@verticalOffset corner: 0@0)
		).
	window 
		addMorph: row
		frame: nominalFractions.

	row on: #mouseEnter send: #paneTransition: to: window.
	row on: #mouseLeave send: #paneTransition: to: window.! !

!TestBrowser methodsFor: 'accessing' stamp: 'tbn 11/5/2004 09:01'!
allTestCaseClasses
	"Returns all test case classes"
	
	^TestCase allSubclasses! !

!TestBrowser methodsFor: 'utilities' stamp: 'minami 11/7/2004 19:05'!
browseFullOnClass: testClass selector: selector browser: browserClass 
	browserClass
		fullOnClass: (self lookupImplementorOfSelector: selector from: testClass)
		selector: selector! !

!TestBrowser methodsFor: 'error list' stamp: 'minami 11/7/2004 19:01'!
browseFullOnSelectedErrorTest

	| selector testCase |
	self selectedErrorTest = 0 ifTrue: [^ self].

	testCase := self errors at: self selectedErrorTest.
	selector := testCase selector.
	self
		browseFullOnClass: testCase class
		selector: selector
		browser: self defaultBrowserClass! !

!TestBrowser methodsFor: 'failure list' stamp: 'minami 11/7/2004 19:01'!
browseFullOnSelectedFailureTest

	| selector testCase |
	self selectedFailureTest = 0 ifTrue: [^ self].

	testCase := self failures at: self selectedFailureTest.
	selector := testCase selector.
	self
		browseFullOnClass: testCase class
		selector: selector
		browser: self defaultBrowserClass! !

!TestBrowser methodsFor: 'error list' stamp: 'minami 11/7/2004 17:04'!
browseSelectedErrorTestByTestBrowser

	| selector testCase |
	self selectedErrorTest = 0 ifTrue: [^ self].

	testCase := self errors at: self selectedErrorTest.
	selector := testCase selector.
	self
		browseFullOnClass: testCase class
		selector: selector
		browser: self class! !

!TestBrowser methodsFor: 'failure list' stamp: 'minami 11/7/2004 18:13'!
browseSelectedFailureTestByTestBrowser

	| selector testCase |
	self selectedFailureTest = 0 ifTrue: [^ self].

	testCase := self failures at: self selectedFailureTest.
	selector := testCase selector.
	self
		browseFullOnClass: testCase class
		selector: selector
		browser: self class! !

!TestBrowser methodsFor: 'interface building-morphic' stamp: 'tbn 11/5/2004 13:06'!
buildBrowserCodeMorph

	^PluggableTextMorph
		on: self
		text: #contents
		accept: #contents:notifying:
		readSelection: #contentsSelection
		menu: #codePaneMenu:shifted:! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:43'!
buildBrowserCodeView

	^PluggableTextView
		on: self
		text: #contents
		accept: #contents:notifying:
		readSelection: #contentsSelection
		menu: #codePaneMenu:shifted:! !

!TestBrowser methodsFor: 'interface building-morphic' stamp: 'tbn 11/5/2004 13:07'!
buildClassListMorph

	^PluggableListMorph
		on: self
		list: #classList
		selected: #classListIndex
		changeSelected: #classListIndex:
		menu: #classListMenu:shifted:
		keystroke: #classListKey:from:! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:43'!
buildClassListView

	| classListView |
	classListView := PluggableListView
				on: self
				list: #classList
				selected: #classListIndex
				changeSelected: #classListIndex:
				menu: #classListMenu:shifted:
				keystroke: #classListKey:from:.
	^classListView! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 13:18'!
buildClassSwitchView

	| aSwitchView |
	aSwitchView := PluggableButtonView
				on: self
				getState: #classMessagesIndicated
				action: #indicateClassMessages.
	aSwitchView 
		label: 'class';
		window: (0 @ 0 extent: 15 @ 15);
		askBeforeChanging: true.
		
	^aSwitchView! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:43'!
buildCommentSwitchView

	| aSwitchView |
	aSwitchView := PluggableButtonView
				on: self
				getState: #classCommentIndicated
				action: #plusButtonHit.
	aSwitchView 
		label: '?' asText allBold;
		borderWidthLeft: 0 right: 1 top: 0 bottom: 0;
		window: (0 @ 0 extent: 10 @ 15);
		askBeforeChanging: true.

	^aSwitchView! !

!TestBrowser methodsFor: 'interface opening-morphic' stamp: 'nsh 7/20/2005 03:13'!
buildDetailsTextMorph
	| row |
	detailsText _ PluggableTextMorph
				on: self
				text: #details
				accept: nil.
	detailsText 
		hideScrollBarsIndefinitely;
		borderWidth: 0;
		hResizing: #spaceFill;
		vResizing: #spaceFill.

	row _ AlignmentMorph newRow
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		layoutInset: 0;
		borderWidth: 0;
		addMorphBack: detailsText.

	^ row! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:43'!
buildDetailsTextView

	^PluggableTextView
		on: self
		text: #details
		accept: nil! !

!TestBrowser methodsFor: 'interface building-morphic' stamp: 'tbn 11/5/2004 10:41'!
buildErrorButtonMorph

	^self privBuildLabelMorph: #errorLabel
! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:43'!
buildErrorButtonView

	| errorButtonView |
	errorButtonView := PluggableButtonView
				on: self
				getState: nil
				action: nil.
	errorButtonView
		 label: 'error(s)';
		 borderWidth: 1;
		 controller: NoController new;
		 insideColor: self defaultLabelColor.
	^errorButtonView! !

!TestBrowser methodsFor: 'interface building-morphic' stamp: 'minami 11/7/2004 18:05'!
buildErrorListMorph

	^PluggableListMorph
		on: self
		list: #errorList
		selected: #selectedErrorTest
		changeSelected: #selectedErrorTest:
		menu: #errorListMenu:! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'minami 11/7/2004 18:05'!
buildErrorsListView

	| errorsList |
	errorsList := PluggableListView
				on: self
				list: #errorList
				selected: #selectedErrorTest
				changeSelected: #selectedErrorTest:
				menu: #errorListMenu:.
	errorsList borderWidth: 1.
	^errorsList! !

!TestBrowser methodsFor: 'interface building-morphic' stamp: 'tbn 11/5/2004 10:41'!
buildFailureButtonMorph

	^self privBuildLabelMorph: #failureLabel! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:43'!
buildFailureButtonView

	| failureButtonView |
	failureButtonView := PluggableButtonView
				on: self
				getState: nil
				action: nil.
	failureButtonView 
		 label: 'failure(s)';
		 borderWidth: 1;
		 controller: NoController new;
		 insideColor: self defaultLabelColor.
	^failureButtonView! !

!TestBrowser methodsFor: 'interface building-morphic' stamp: 'minami 11/7/2004 18:06'!
buildFailuresListMorph

	^PluggableListMorph
		on: self
		list: #failureList
		selected: #selectedFailureTest
		changeSelected: #selectedFailureTest:
		menu: #failureListMenu:! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'minami 11/7/2004 18:06'!
buildFailuresListView

	| failuresList |
	failuresList := PluggableListView
				on: self
				list: #failureList
				selected: #selectedFailureTest
				changeSelected: #selectedFailureTest:
				menu: #failureListMenu:.
	failuresList borderWidth: 1.
	^failuresList! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:44'!
buildInstanceClassSwitchView

	| aView aSwitchView instSwitchView comSwitchView |
	aView := View new model: self.
	instSwitchView := self buildInstanceSwitchView.
	comSwitchView := self buildCommentSwitchView.
	aSwitchView := self buildClassSwitchView.

	aView addSubView: instSwitchView.
	aView addSubView: comSwitchView toRightOf: instSwitchView.
	aView addSubView: aSwitchView toRightOf: comSwitchView.
	aView borderWidth: 1.

	^aView! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:44'!
buildInstanceSwitchView

	| aSwitchView |
	aSwitchView := PluggableButtonView
				on: self
				getState: #instanceMessagesIndicated
				action: #indicateInstanceMessages.
	aSwitchView 
		label: 'instance';
		borderWidthLeft: 0 right: 1 top: 0 bottom: 0;
		window: (0 @ 0 extent: 25 @ 15);
		askBeforeChanging: true.

	^aSwitchView! !

!TestBrowser methodsFor: 'interface building-morphic' stamp: 'tbn 11/5/2004 10:40'!
buildMessageCategoryListMorph

	^PluggableListMorph
		on: self
		list: #messageCategoryList
		selected: #messageCategoryListIndex
		changeSelected: #messageCategoryListIndex:
		menu: #messageCategoryMenu:! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:44'!
buildMessageCategoryListView

	| messageCategoryListView |
	messageCategoryListView := PluggableListView
				on: self
				list: #messageCategoryList
				selected: #messageCategoryListIndex
				changeSelected: #messageCategoryListIndex:
				menu: #messageCategoryMenu:.

	^messageCategoryListView! !

!TestBrowser methodsFor: 'interface building-morphic' stamp: 'tbn 11/5/2004 10:41'!
buildMessageListMorph

	^PluggableListMorph
		on: self
		list: #messageList
		selected: #messageListIndex
		changeSelected: #messageListIndex:
		menu: #messageListMenu:shifted:
		keystroke: #messageListKey:from:! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:44'!
buildMessageListView

	| messageListView |
	messageListView := PluggableListView
				on: self
				list: #messageList
				selected: #messageListIndex
				changeSelected: #messageListIndex:
				menu: #messageListMenu:shifted:
				keystroke: #messageListKey:from:.
	messageListView menuTitleSelector: #messageListSelectorTitle.
	^messageListView! !

!TestBrowser methodsFor: 'interface opening-morphic' stamp: 'nsh 7/20/2005 03:13'!
buildPassFailTextMorph
	| row |
	passFailText _ PluggableTextMorph
				on: self
				text: #passFail
				accept: nil.
	passFailText 
		hideScrollBarsIndefinitely;
		borderWidth: 0;
		hResizing: #spaceFill;
		vResizing: #spaceFill.

	row _ AlignmentMorph newRow
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		layoutInset: 0;
		borderWidth: 0;
		addMorphBack: passFailText.

	^ row! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:44'!
buildPassFailTextView

	^PluggableTextView
		on: self
		text: #passFail
		accept: nil! !

!TestBrowser methodsFor: 'interface building-morphic' stamp: 'tbn 11/5/2004 13:19'!
buildRefreshButtonMorph

	| refreshButton row |
	refreshButton := PluggableButtonMorph
				on: self
				getState: #refreshButtonState
				action: #refreshTests
				label: #refreshButtonLabel.
	refreshButton
		onColor: self runButtonColor offColor: self runButtonColor;
		borderWidth: 0;
		hResizing: #spaceFill;
		vResizing: #spaceFill.

	row := AlignmentMorph newRow
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		layoutInset: 0;
		borderWidth: 0;
		addMorphBack: refreshButton.

	^row! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:44'!
buildRefreshButtonView

	| refreshButton |
	refreshButton := PluggableButtonView
				on: self
				getState: #refreshButtonState
				action: #refreshTests
				label: #refreshButtonLabel.
	refreshButton
		label: self refreshButtonLabel;
		clearInside: self runButtonColor;
		insideColor: self runButtonColor;
		borderWidth: 1.
	^refreshButton! !

!TestBrowser methodsFor: 'interface building-morphic' stamp: 'tbn 11/5/2004 10:41'!
buildRunButtonMorph

	^self
		privBuildButtonMorphOn: self
		getState: #runButtonState
		action: #runAllTests
		label: #runButtonLabel
		color: self runButtonColor! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:44'!
buildRunButtonView

	| runButton |
	runButton := PluggableButtonView
				on: self
				getState: #runButtonState
				action: #runAllTests
				label: #runButtonLabel.
	runButton 
		label: self runButtonLabel;
		insideColor: self runButtonColor;
		borderWidth: 1.

	^runButton! !

!TestBrowser methodsFor: 'interface building-morphic' stamp: 'tbn 11/5/2004 10:41'!
buildRunCategoryButtonMorph

	^self
		privBuildButtonMorphOn: self
		getState: #runButtonState
		action: #runCategoryTest
		label: #runCategoryButtonLabel
		color: self runButtonColor! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:44'!
buildRunCategoryButtonView

	| runCategoryButton |
	runCategoryButton := PluggableButtonView
				on: self
				getState: #runButtonState
				action: #runCategoryTest
				label: #runCategoryButtonLabel.
	runCategoryButton 
		 label: self runCategoryButtonLabel;
		 clearInside: self runButtonColor;
		 insideColor: self runButtonColor;
		 borderWidth: 1.
	^runCategoryButton! !

!TestBrowser methodsFor: 'interface building-morphic' stamp: 'tbn 11/5/2004 10:42'!
buildRunClassButtonMorph

	^self
		privBuildButtonMorphOn: self
		getState: #runButtonState
		action: #runClassTest
		label: #runClassButtonLabel
		color: self runButtonColor! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:45'!
buildRunClassButtonView

	| runClassButton |
	runClassButton := PluggableButtonView
				on: self
				getState: #runButtonState
				action: #runClassTest
				label: #runClassButtonLabel.
	runClassButton
		 label: self runClassButtonLabel;
		 clearInside: self runButtonColor;
		 insideColor: self runButtonColor;
		 borderWidth: 1.
	^runClassButton! !

!TestBrowser methodsFor: 'interface building-morphic' stamp: 'tbn 11/5/2004 13:19'!
buildRunMethodButtonMorph

	^self
		privBuildButtonMorphOn: self
		getState: #runButtonState
		action: #runMethodTest
		label: #runMethodButtonLabel
		color: self runButtonColor! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:45'!
buildRunMethodButtonView

	| runMethodButton |
	runMethodButton := PluggableButtonView
				on: self
				getState: #runButtonState
				action: #runMethodTest
				label: #runMethodButtonLabel.
	runMethodButton
		 label: self runMethodButtonLabel;
		 clearInside: self runButtonColor;
		 insideColor: self runButtonColor;
		 borderWidth: 1.
	^runMethodButton! !

!TestBrowser methodsFor: 'interface building-morphic' stamp: 'tbn 11/5/2004 10:42'!
buildSystemCategoryListMorph

	^PluggableListMorph
		on: self
		list: #systemCategoryList
		selected: #systemCategoryListIndex
		changeSelected: #systemCategoryListIndex:
		menu: #systemCategoryMenu:
		keystroke: #systemCatListKey:from:! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:45'!
buildSystemCategoryListView

	| systemCategoryListView |
	systemCategoryListView := PluggableListView
				on: self
				list: #systemCategoryList
				selected: #systemCategoryListIndex
				changeSelected: #systemCategoryListIndex:
				menu: #systemCategoryMenu:
				keystroke: #systemCatListKey:from:.

	^systemCategoryListView! !

!TestBrowser methodsFor: 'interface opening-morphic' stamp: 'nsh 7/20/2005 03:13'!
buildTemplateMorph
	| templateMorph |
	templateMorph _ PluggableTextMorph
				on: self
				text: #pattern
				accept: #pattern:.
	templateMorph acceptOnCR: true.
	templateMorph hideScrollBarsIndefinitely.
	^ templateMorph! !

!TestBrowser methodsFor: 'interface building-mvc' stamp: 'tbn 11/5/2004 10:45'!
buildTemplateView

	| templateView |
	templateView := PluggableTextView
				on: self
				text: #pattern
				accept: #pattern:.
	templateView askBeforeDiscardingEdits: false.

	^templateView! !

!TestBrowser methodsFor: 'interface building-morphic' stamp: 'tbn 11/5/2004 10:42'!
buildToggleFilteringButtonMorph

	^self
		privBuildButtonMorphOn: self
		getState: #toggleButtonState
		action: #toggleFiltering
		label: #toggleFilteringButtonLabel
		color: self toggleButtonColor! !

!TestBrowser methodsFor: 'interface building-morphic' stamp: 'tbn 11/5/2004 10:42'!
buildToggleTestPaneButtonMorph

	^self
		privBuildButtonMorphOn: self
		getState: #toggleButtonState
		action: #toggleTestPane
		label: #toggleTestPaneButtonLabel
		color: self toggleButtonColor! !

!TestBrowser methodsFor: 'updating' stamp: 'minami 11/7/2004 17:32'!
changed: anAspect
	anAspect == #systemCategoryList ifTrue: [systemCategoryList := nil].
	super changed: anAspect! !

!TestBrowser methodsFor: 'class list' stamp: 'tbn 11/5/2004 13:08'!
classList

	^systemCategoryListIndex = 0
		ifTrue: [Array new]
		ifFalse: [self classListAtCategoryNamed: (self systemCategoryList at: systemCategoryListIndex)]! !

!TestBrowser methodsFor: 'class list' stamp: 'tbn 11/5/2004 13:08'!
classListAtCategoryNamed: aCategoryName 

	| classNames |
	classNames := systemOrganizer listAtCategoryNamed: aCategoryName.
	^filterMode 
		ifFalse: [classNames]
		ifTrue: [classNames intersection: (self allTestCaseClasses collect: [:each | each name])]! !

!TestBrowser methodsFor: 'processing' stamp: 'minami 11/7/2004 15:11'!
clearTestHolder
	self testHolder clear.! !

!TestBrowser methodsFor: 'error list' stamp: 'tbn 11/5/2004 13:11'!
debugErrorTest

	self debugErrorTest: self selectedErrorTest! !

!TestBrowser methodsFor: 'processing' stamp: 'minami 11/7/2004 19:07'!
debugErrorTest: anInteger 
	self selectedErrorTest: anInteger.
	anInteger ~= 0 ifTrue: [(self errors at: anInteger) debug]! !

!TestBrowser methodsFor: 'failure list' stamp: 'tbn 11/5/2004 13:12'!
debugFailureTest

	self debugFailureTest: self selectedFailureTest! !

!TestBrowser methodsFor: 'processing' stamp: 'minami 11/7/2004 19:07'!
debugFailureTest: anInteger 
	self selectedFailureTest: anInteger.
	anInteger ~= 0 ifTrue: [(self failures at: anInteger) debug]! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:09'!
debugState

	^true! !

!TestBrowser methodsFor: 'defaults' stamp: 'tbn 11/5/2004 13:11'!
defaultBackgroundColor
	"Returns the receivers default background color"
	
	^Color white! !

!TestBrowser methodsFor: 'defaults' stamp: 'minami 11/7/2004 19:01'!
defaultBrowserClass
	^ Smalltalk at: #Browser
! !

!TestBrowser methodsFor: 'defaults' stamp: 'tbn 11/5/2004 10:05'!
defaultBrowserTitle
	"Returns the receivers default browser title"
	
	^self class version! !

!TestBrowser methodsFor: 'defaults' stamp: 'tbn 11/5/2004 13:11'!
defaultLabelColor
	"Returns the receivers default label color"

	^Color veryVeryLightGray! !

!TestBrowser methodsFor: 'accessing' stamp: 'tbn 11/5/2004 13:07'!
details
	"Returns the receivers detail information"
	
	^details! !

!TestBrowser methodsFor: 'updating' stamp: 'minami 11/7/2004 17:32'!
displayDetails: aString 
	details := aString.
	self changed: #details! !

!TestBrowser methodsFor: 'updating' stamp: 'minami 11/7/2004 17:32'!
displayPassFail: aString 
	passFail := aString.
	self changed: #passFail! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:09'!
errorColor

	^Color red! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:09'!
errorLabel

	^'error(s)' copy! !

!TestBrowser methodsFor: 'accessing' stamp: 'minami 11/7/2004 18:10'!
errorList
	^ self errors collect: [:each | self printTestCase: each]! !

!TestBrowser methodsFor: 'error list' stamp: 'tbn 11/5/2004 13:15'!
errorListMenu: aMenu

	^aMenu 
		labels: #('debug test' 'browse full (system browser)' 'browse full (test browser)')
		lines: #(1)
		selections: #(
			debugErrorTest
			browseFullOnSelectedErrorTest
			browseSelectedErrorTestByTestBrowser)! !

!TestBrowser methodsFor: 'accessing' stamp: 'minami 11/7/2004 17:03'!
errors
	"Returns the receivers errors"
	
	^ self testHolder errors! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:09'!
failColor

	^Color yellow! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:09'!
failureLabel

	^'failure(s)' copy! !

!TestBrowser methodsFor: 'accessing' stamp: 'minami 11/7/2004 18:10'!
failureList
	^ self failures collect: [:each | self printTestCase: each]! !

!TestBrowser methodsFor: 'failure list' stamp: 'tbn 11/5/2004 13:15'!
failureListMenu: aMenu

	^aMenu 
		labels: #('debug test' 'browse full (system browser)' 'browse full (test browser)')
		lines: #(1)
		selections: #(
		  debugFailureTest
		  browseFullOnSelectedFailureTest
		  browseSelectedFailureTestByTestBrowser)! !

!TestBrowser methodsFor: 'accessing' stamp: 'minami 11/7/2004 17:03'!
failures
	"Returns the receivers failures"
	
	^ self testHolder failures! !

!TestBrowser methodsFor: 'testing' stamp: 'minami 11/7/2004 19:10'!
hasPassed
	^ self result hasPassed! !

!TestBrowser methodsFor: 'initialize-release' stamp: 'minami 11/7/2004 17:38'!
initialize
	"Initializes the receiver"
	
	super initialize.

	passFail := 'N/A'.
	details := '...'.
	selectedFailureTest := 0.
	selectedErrorTest := 0.
	systemCategoryList := nil.
	testPaneVisible := true.
	filterMode := true.

	testHolder := TBTestHolder new.! !

!TestBrowser methodsFor: 'initialize-release' stamp: 'tbn 11/5/2004 13:17'!
initializeIndex

	systemCategoryListIndex := 0.
	classListIndex := 0.
	messageCategoryListIndex := 0.
	messageListIndex := 0.
	metaClassIndicated := false.
	editSelection := #none! !

!TestBrowser methodsFor: 'testing' stamp: 'minami 11/7/2004 15:21'!
isEmptySuite
	^ self testHolder isEmptySuite! !

!TestBrowser methodsFor: 'testing' stamp: 'minami 11/7/2004 15:11'!
isTestCaseClass: aClass
	"Returns true if the class is a valid test case class"
	
	^ self testHolder isTestCaseClass: aClass! !

!TestBrowser methodsFor: 'initialize-release' stamp: 'tbn 11/5/2004 13:16'!
labelString

	^self selectedClass
		ifNil: [self defaultBrowserTitle]
		ifNotNil: [self defaultBrowserTitle , ': ' , self selectedClass printString]! !

!TestBrowser methodsFor: 'interface opening-morphic' stamp: 'tbn 11/5/2004 10:42'!
layoutWithoutTestPane

	topWindowMorph removePanes.

	topWindowMorph 
		addMorph: self buildSystemCategoryListMorph frame: (0.0 @ 0.0 corner: 0.25 @ 0.2);
		addMorph: self buildTemplateMorph frame: (0.0 @ 0.20 corner: 0.25 @ 0.25);
		addMorph: self buildToggleFilteringButtonMorph frame: (0.0 @ 0.25 corner: 0.12 @ 0.3);
		addMorph: self buildToggleTestPaneButtonMorph frame: (0.12 @ 0.25 corner: 0.25 @ 0.3);
		addMorph: self buildClassListMorph frame: (0.25 @ 0.0 corner: 0.5 @ 0.25);
		addMorph: self buildMorphicSwitches frame: (0.25 @ 0.25 corner: 0.5 @ 0.3);
		addMorph: self buildMessageCategoryListMorph frame: (0.5 @ 0.0 corner: 0.75 @ 0.3);
		addMorph: self buildMessageListMorph frame: (0.75 @ 0.0 corner: 1 @ 0.3).

	self 
		addLowerPanesTo: topWindowMorph 
		at: (0@0.3 corner: 1@1).

	self refreshWindow.

	^ topWindowMorph! !

!TestBrowser methodsFor: 'interface opening-morphic' stamp: 'minami 11/7/2004 18:32'!
layoutWithTestPane

	topWindowMorph removePanes.

	topWindowMorph 
		addMorph: self buildSystemCategoryListMorph frame: (0.0 @ 0.0 extent: 0.25 @ 0.15);
		addMorph: self buildTemplateMorph frame: (0.0 @ 0.15 extent: 0.25 @ 0.05);
		addMorph: self buildToggleFilteringButtonMorph frame: (0.0 @ 0.20 extent: 0.12 @ 0.05);
		addMorph: self buildToggleTestPaneButtonMorph frame: (0.12 @ 0.20 extent: 0.13 @ 0.05);
		addMorph: self buildClassListMorph frame: (0.25 @ 0.0 extent: 0.25 @ 0.20);
		addMorph: self buildMorphicSwitches frame: (0.25 @ 0.2 extent: 0.25 @ 0.05);
		addMorph: self buildMessageCategoryListMorph frame: (0.5 @ 0.0 extent: 0.25 @ 0.25);
		addMorph: self buildMessageListMorph frame: (0.75 @ 0.0 extent: 0.25 @ 0.25).

	topWindowMorph 
		addMorph: self buildRefreshButtonMorph frame: (0.0 @ 0.25 extent: 0.17 @ 0.12 "0.08");
		addMorph: self buildRunButtonMorph frame: (0.0 @ 0.37 "0.33" extent: 0.17 @ 0.12 "0.08");
		"addMorph: self buildStopButtonMorph frame: (0.0 @ 0.41 extent: 0.17 @ 0.08);"
		addMorph: self buildFailureButtonMorph frame: (0.17 @ 0.25 extent: 0.33 @ 0.05);
		addMorph: self buildFailuresListMorph frame: (0.17 @ 0.30 extent: 0.33 @ 0.19);
		addMorph: self buildErrorButtonMorph frame: (0.50 @ 0.25 extent: 0.33 @ 0.05);
		addMorph: self buildErrorListMorph frame: (0.50 @ 0.30 extent: 0.33 @ 0.19);
		addMorph: self buildRunCategoryButtonMorph frame: (0.83 @ 0.25 extent: 0.17 @ 0.08);
		addMorph: self buildRunClassButtonMorph frame: (0.83 @ 0.33 extent: 0.17 @ 0.08);
		addMorph: self buildRunMethodButtonMorph frame: (0.83 @ 0.41 extent: 0.17 @ 0.08).

	topWindowMorph 
		addMorph: self buildPassFailTextMorph frame: (0.0 @ 0.49 extent: 0.30 @ 0.05);
		addMorph: self buildDetailsTextMorph frame: (0.30 @ 0.49 extent: 0.70 @ 0.05).
	
	topWindowMorph 
		addMorph: self buildBrowserCodeMorph frame: (0.0 @ 0.54 extent: 1 @ 0.46).

	topWindowMorph setUpdatablePanesFrom: #(systemCategoryList classList messageCategoryList messageList).
	self refreshWindow.

	self isEmptySuite ifFalse: [self updateWindow].

	^ topWindowMorph! !

!TestBrowser methodsFor: 'utilities' stamp: 'tbn 11/5/2004 13:17'!
lookupImplementorOfSelector: aSelector from: aClass

	| lookupClass |
	lookupClass := aClass.
	[lookupClass isNil]
		whileFalse: [(lookupClass includesSelector: aSelector)
				ifTrue: [^ lookupClass].
			lookupClass := lookupClass superclass].
		
	self error: (aClass name , '>>' , aSelector , ' ==> method has not found.')! !

!TestBrowser methodsFor: 'error list' stamp: 'tbn 11/5/2004 13:22'!
notATestCaseClass

	self runWindow.
	self displayPassFail: 
		(Preferences testBrowserRunsAbstractClasses 
			ifTrue: ['Not a test class']
			ifFalse: ['Abstract or not a test class'])! !

!TestBrowser methodsFor: 'processing' stamp: 'minami 11/7/2004 19:07'!
notAvailable
	self runWindow.
	self displayPassFail: 'No Test Suite Selected'! !

!TestBrowser methodsFor: 'interface opening-morphic' stamp: 'tbn 11/5/2004 10:42'!
openAsMorphEditing: aString

	Smalltalk isMorphic ifFalse: [^ self openEditString: aString].

	topWindowMorph := (SystemWindow labelled: aString)
							 model: self;
							 yourself.

	self layoutWithTestPane.
	
	^topWindowMorph! !

!TestBrowser methodsFor: 'interface opening-mvc' stamp: 'minami 11/7/2004 23:17'!
openEditString: aString 
	"Create a pluggable version of all the views for a Browser, including views and controllers."

	| systemCategoryListView classListView messageCategoryListView messageListView browserCodeView topView switchView refreshButton runButton failuresList errorsList runCategoryButton runClassButton runMethodButton failureButtonView errorButtonView templateView "stopButton" |

	Smalltalk isMorphic ifTrue: [^ self openAsMorphEditing: aString].


	topView := (StandardSystemView new model: self)
				borderWidth: 1;
				yourself.


	systemCategoryListView := self buildSystemCategoryListView.
	systemCategoryListView window: (0 @ 0 extent: 50 @ 45).
	topView addSubView: systemCategoryListView.

	templateView := self buildTemplateView.
	templateView window: (0 @ 0 extent: 50 @ 15).
	topView addSubView: templateView below: systemCategoryListView.

	classListView := self buildClassListView.
	classListView window: (0 @ 0 extent: 50 @ 45).
	topView addSubView: classListView toRightOf: systemCategoryListView.

	switchView := self buildInstanceClassSwitchView.
	switchView window: (0 @ 0 extent: 50 @ 15).
	topView addSubView: switchView below: classListView.

	messageCategoryListView := self buildMessageCategoryListView.
	messageCategoryListView window: (0 @ 0 extent: 50 @ 60).
	topView addSubView: messageCategoryListView toRightOf: classListView.

	messageListView := self buildMessageListView.
	messageListView window: (0 @ 0 extent: 50 @ 60).
	topView addSubView: messageListView toRightOf: messageCategoryListView.

	refreshButton := self buildRefreshButtonView.
	refreshButton window: (0 @ 0 corner: 35 @ 30 "20").
	topView addSubView: refreshButton below: templateView.

	runButton := self buildRunButtonView.
	runButton window: (0 @ 0 corner: 35 @ 30 "20").
	topView addSubView: runButton below: refreshButton.

	failureButtonView := self buildFailureButtonView.
	failureButtonView window: (0 @ 0 extent: 65 @ 10).
	topView addSubView: failureButtonView toRightOf: refreshButton.

	failuresList := self buildFailuresListView.
	failuresList window: (0 @ 0 corner: 65 @ 50).
	topView addSubView: failuresList below: failureButtonView.

	errorButtonView := self buildErrorButtonView.
	errorButtonView window: (0 @ 0 extent: 65 @ 10).
	topView addSubView: errorButtonView toRightOf: failureButtonView.

	errorsList := self buildErrorsListView.
	errorsList window: (0 @ 0 corner: 65 @ 50).
	topView addSubView: errorsList below: errorButtonView.

	runCategoryButton := self buildRunCategoryButtonView.
	runCategoryButton window: (0 @ 0 corner: 35 @ 20).
	topView addSubView: runCategoryButton toRightOf: errorButtonView.

	runClassButton := self buildRunClassButtonView.
	runClassButton window: (0 @ 0 corner: 35 @ 20).
	topView addSubView: runClassButton below: runCategoryButton.

	runMethodButton := self buildRunMethodButtonView.
	runMethodButton window: (0 @ 0 corner: 35 @ 20).
	topView addSubView: runMethodButton below: runClassButton.

	"passFailText is instance variable"
	passFailText := self buildPassFailTextView.
	passFailText window: (0 @ 0 extent: 70 @ 15).
	topView addSubView: passFailText below: runButton.
	
	"detailsText is instance variable"
	detailsText := self buildDetailsTextView.
	detailsText window: (0 @ 0 extent: 130 @ 15).
	topView addSubView: detailsText toRightOf: passFailText.

	browserCodeView := self buildBrowserCodeView.
	browserCodeView window: (0 @ 0 extent: 200 @ 110).
	topView addSubView: browserCodeView below: passFailText.

	aString ifNotNil: [
		browserCodeView editString: aString.
		browserCodeView hasUnacceptedEdits: true].

	topView setUpdatablePanesFrom: #(#systemCategoryList #classList #messageCategoryList #messageList ).

	^topView! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:09'!
passColor

	^Color green! !

!TestBrowser methodsFor: 'accessing' stamp: 'tbn 11/5/2004 13:08'!
passFail

	^passFail! !

!TestBrowser methodsFor: 'accessing' stamp: 'tbn 11/5/2004 13:08'!
pattern
	"Returns the receivers filter pattern"
	
	(pattern isNil or: [pattern isEmpty]) ifTrue: [pattern := '*'].
	^pattern! !

!TestBrowser methodsFor: 'accessing' stamp: 'tbn 11/5/2004 13:08'!
pattern: textOrStringOrNil 
	"Sets the receivers filter pattern"
		
	(textOrStringOrNil = self pattern) ifTrue: [^ true].

	textOrStringOrNil
		ifNil: [pattern := '*']
		ifNotNil: [pattern := textOrStringOrNil asString].

	self systemCategoryListIndex: 0.

	self initializeIndex.
	self changed: #systemCategoryList.
	self changed: #classList.
	self changed: #messageCategoryList.
	self changed: #messageList.
	contents := nil.
	self contentsChanged.
	
	^true! !

!TestBrowser methodsFor: 'utilities' stamp: 'tbn 11/5/2004 13:17'!
printTestCase: aTestCase

	| ws testClass selector lookupClass |
	testClass := aTestCase class.
	selector := aTestCase selector.

	ws := WriteStream on: String new.

	ws nextPutAll: testClass printString.
	
	((lookupClass := self lookupImplementorOfSelector: selector from: testClass ) == testClass) ifFalse: [
		ws nextPutAll: '('.
		ws nextPutAll: lookupClass printString.
		ws nextPutAll: ')'.		
	].
	
	ws nextPutAll: '>>#'.
	ws nextPutAll: selector.

	^ws contents! !

!TestBrowser methodsFor: 'interface opening-morphic' stamp: 'tbn 11/5/2004 10:42'!
privBuildButtonMorphOn: anObject getState: stateSymbol action: actionSymbol label: labelSymbol color: aColor

	| button row |
	button := PluggableButtonMorph
				on: anObject
				getState: stateSymbol
				action: actionSymbol
				label: labelSymbol.
	button 
		onColor: aColor offColor: aColor;
		borderWidth: 0;
		hResizing: #spaceFill;
		vResizing: #spaceFill.

	row := AlignmentMorph newRow
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		layoutInset: 0;
		borderWidth: 0;
		addMorphBack: button.

	^row! !

!TestBrowser methodsFor: 'interface opening-morphic' stamp: 'tbn 11/5/2004 10:43'!
privBuildLabelMorph: aSymbol

	| labelMorph row |
	labelMorph := PluggableButtonMorph
				on: nil
				getState: nil
				action: nil.
	labelMorph
		label: (self perform: aSymbol);
		onColor: self defaultLabelColor offColor: self defaultLabelColor;
		feedbackColor: Color transparent;
		borderWidth: 0;
		hResizing: #spaceFill;
		vResizing: #spaceFill.

	row := AlignmentMorph newRow
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		layoutInset: 0;
		borderWidth: 0;
		addMorphBack: labelMorph.

	^row! !

!TestBrowser methodsFor: 'processing' stamp: 'minami 11/7/2004 15:11'!
privRunTests
	self testHolder runSuite.
! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:09'!
refreshButtonLabel

	^'REFRESH' copy! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:09'!
refreshButtonState

	^true! !

!TestBrowser methodsFor: 'processing' stamp: 'minami 11/7/2004 19:07'!
refreshTests
	self clearTestHolder.

	self tests: (self allTestCaseClasses collect: [:each | each name]) asOrderedCollection.
	self selectedErrorTest: 0.
	self selectedFailureTest: 0.
	self refreshWindow! !

!TestBrowser methodsFor: 'updating' stamp: 'minami 11/7/2004 17:34'!
refreshWindow
	self updatePartColors: Color white.
	self updateErrors.
	self updateFailures.
	self displayDetails: '...'.
	self displayPassFail: 'N/A'.! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:09'!
resetColor

	^Color white! !

!TestBrowser methodsFor: 'accessing' stamp: 'minami 11/7/2004 17:23'!
result
	^ self testHolder result! !

!TestBrowser methodsFor: 'processing' stamp: 'minami 11/7/2004 15:48'!
runAllTests
	(self systemCategoryList isEmpty) ifTrue: [^ self notAvailable].
	self runTests: [self suiteAll].! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:09'!
runButtonColor

	^Color lightGray! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:09'!
runButtonLabel

	^'RUN ALL' copy! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:09'!
runButtonState

	^true! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:10'!
runCategoryButtonLabel

	^'RUN(CATEGORY)' copy! !

!TestBrowser methodsFor: 'processing' stamp: 'minami 11/7/2004 19:08'!
runCategoryTest
	(self systemCategoryListIndex == 0) ifTrue: [^ self notAvailable].

	self runTests: [self suiteCategory]
! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:10'!
runClassButtonLabel

	^'RUN(CLASS)' copy! !

!TestBrowser methodsFor: 'processing' stamp: 'minami 11/7/2004 18:58'!
runClassTest
	self classListIndex = 0 ifTrue: [^ self notAvailable].
	(self isTestCaseClass: self selectedClass) ifFalse: [^ self abstractOrNotATest].

	self runTests: [self suiteClass].
! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:10'!
runMethodButtonLabel

	^'RUN(METHOD)' copy! !

!TestBrowser methodsFor: 'processing' stamp: 'minami 11/7/2004 18:29'!
runMethodTest
	(self messageListIndex == 0) ifTrue: [^ self notAvailable].
	(self isTestCaseClass: self selectedClass) ifFalse: [^ self abstractOrNotATest].

	self runTests: [self suiteMethod]
! !

!TestBrowser methodsFor: 'processing' stamp: 'minami 11/7/2004 15:26'!
runTests: suiteBlock
	suiteBlock value.
	self isEmptySuite ifTrue: [^ self notAvailable].

	self runWindow.
	self privRunTests.
	self updateWindow.

	self selectedErrorTest: 0.
	self selectedFailureTest: 0.! !

!TestBrowser methodsFor: 'updating' stamp: 'minami 11/7/2004 17:33'!
runWindow
	self updatePartColors: Color white.
	self updateErrors.
	self updateFailures.
	self displayDetails: '...'.
	self displayPassFail: 'Running...'.! !

!TestBrowser methodsFor: 'error list' stamp: 'tbn 11/5/2004 13:11'!
selectedErrorTest

	(self errors isNil or: [self errors isEmpty]) ifTrue: [^ 0].
	^selectedErrorTest! !

!TestBrowser methodsFor: 'error list' stamp: 'tbn 11/5/2004 13:11'!
selectedErrorTest: anInteger

	selectedErrorTest := anInteger.
	self changed: #selectedErrorTest! !

!TestBrowser methodsFor: 'failure list' stamp: 'tbn 11/5/2004 13:12'!
selectedFailureTest

	(self failures isNil or: [self failures isEmpty]) ifTrue: [^ 0].
	^selectedFailureTest! !

!TestBrowser methodsFor: 'failure list' stamp: 'tbn 11/5/2004 13:12'!
selectedFailureTest: anInteger

	selectedFailureTest := anInteger.
	self changed: #selectedFailureTest! !

!TestBrowser methodsFor: 'initialize-release' stamp: 'tbn 11/5/2004 13:16'!
setCategory: aCategory

	| systemCatIndex |
	systemCatIndex := self systemCategoryList indexOf: aCategory ifAbsent: [0].
	self systemCategoryListIndex: systemCatIndex.
! !

!TestBrowser methodsFor: 'initialize-release' stamp: 'tbn 11/5/2004 13:16'!
setClass: aBehavior selector: aSymbol

	| isMeta aClass systemCatIndex messageCatIndex |
	aBehavior ifNil: [^ self].

	(aBehavior isKindOf: Metaclass)
		ifTrue: [isMeta := true. aClass := aBehavior soleInstance]
		ifFalse: [isMeta := false. aClass := aBehavior].
	systemCatIndex := self systemCategoryList indexOf: aBehavior category ifAbsent: [0].
	self systemCategoryListIndex: systemCatIndex.
	self classListIndex: ((self classListAtCategoryNamed: (self systemCategoryList at: systemCatIndex)) indexOf: aClass name).
	self metaClassIndicated: isMeta.
	aSymbol ifNil: [^ self].
	messageCatIndex := aBehavior organization numberOfCategoryOfElement: aSymbol.
	self messageCategoryListIndex: messageCatIndex + 1. "<- FIXED offset"
	messageCatIndex = 0 ifTrue: [^ self].
	self messageListIndex: ((aBehavior organization listAtCategoryNumber: messageCatIndex) indexOf: aSymbol)! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:10'!
stopButtonLabel

	^'STOP' copy! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:10'!
stopButtonState

	^true! !

!TestBrowser methodsFor: 'processing' stamp: 'minami 11/7/2004 15:50'!
suiteAll
	| classes |
	classes := self allTestCaseClasses 
				select: [:aClass | self pattern match: aClass category asString].
	self testHolder buildSuiteClasses: classes! !

!TestBrowser methodsFor: 'processing' stamp: 'minami 11/7/2004 18:35'!
suiteCategory
	| classes |
	classes := OrderedCollection new.
	self classList
		do: [:each | classes add: (Smalltalk at: each)].
	self testHolder buildSuiteClasses: classes! !

!TestBrowser methodsFor: 'processing' stamp: 'minami 11/7/2004 15:43'!
suiteClass
	self testHolder buildSuiteClass: self selectedClass! !

!TestBrowser methodsFor: 'processing' stamp: 'minami 11/7/2004 15:35'!
suiteMethod
	self testHolder buildSuiteClass: self selectedClass method: self selectedMessageName! !

!TestBrowser methodsFor: 'category list' stamp: 'tbn 11/5/2004 13:08'!
systemCategoryList

	^filterMode 
		ifFalse: 
			[systemCategoryList := (systemOrganizer categories 
						select: [:each | self pattern match: each]) asArray]
		ifTrue: 
			[systemCategoryList ifNil: 
					[| testCategories |
					testCategories := self allTestCaseClasses collect: [:each | each category].
					systemCategoryList := (testCategories 
								select: [:each | self pattern match: each]) asArray]
				ifNotNil: [systemCategoryList]]! !

!TestBrowser methodsFor: 'category list' stamp: 'tbn 11/5/2004 13:16'!
systemCategoryMenu: aMenu

	^ filterMode 
		ifFalse: [super systemCategoryMenu: aMenu]
		ifTrue: [aMenu 
				labels: #('browse all' 'browse' 'update' 'rename...' 'remove')
				lines: #(2 4)
				selections: #( 
					browseAllClasses 
					buildSystemCategoryBrowser
					updateSystemCategories
		 			renameSystemCategory 
					removeSystemCategory )]! !

!TestBrowser methodsFor: 'accessing' stamp: 'minami 11/7/2004 15:11'!
testHolder
	^ testHolder! !

!TestBrowser methodsFor: 'accessing' stamp: 'tbn 11/5/2004 13:08'!
tests
	"Returns the receivers tests"
	
	^tests! !

!TestBrowser methodsFor: 'accessing' stamp: 'minami 11/7/2004 18:55'!
tests: anOrderedCollection
	"Sets the receivers tests to the given collection"
	
	tests := anOrderedCollection.
	self changed: #tests.! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:10'!
toggleButtonColor

	^Color veryVeryLightGray! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:10'!
toggleButtonState

	^true! !

!TestBrowser methodsFor: 'interface opening-morphic' stamp: 'tbn 11/5/2004 10:43'!
toggleFiltering

	filterMode := filterMode not.
	testPaneVisible
		ifTrue: [self layoutWithTestPane]	
		ifFalse: [self layoutWithoutTestPane].

	self systemCategoryListIndex: 0.

	self initializeIndex.
	self changed: #systemCategoryList.
	self changed: #classList.
	self changed: #messageCategoryList.
	self changed: #messageList.
	contents := nil.
	self contentsChanged.! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:10'!
toggleFilteringButtonLabel

	^filterMode
		ifTrue: ['tests->all' copy]
		ifFalse: ['all->tests' copy]! !

!TestBrowser methodsFor: 'interface opening-morphic' stamp: 'tbn 11/5/2004 10:43'!
toggleTestPane

	testPaneVisible
		ifTrue: [
			self layoutWithoutTestPane.
			testPaneVisible := false]
		ifFalse: [
			self layoutWithTestPane.
			testPaneVisible := true]! !

!TestBrowser methodsFor: 'constants' stamp: 'tbn 11/5/2004 13:10'!
toggleTestPaneButtonLabel

	^'show/hide' copy! !

!TestBrowser methodsFor: 'updating' stamp: 'minami 11/7/2004 17:29'!
updateDetails 
	self displayDetails: self testHolder resultString! !

!TestBrowser methodsFor: 'updating' stamp: 'minami 11/7/2004 18:05'!
updateErrors
	self changed: #errorList! !

!TestBrowser methodsFor: 'updating' stamp: 'minami 11/7/2004 18:11'!
updateFailures
	self changed: #failureList! !

!TestBrowser methodsFor: 'updating' stamp: 'minami 11/7/2004 17:33'!
updatePartColors: aColor
	passFailText isMorph
		ifTrue: [
			passFailText color: aColor.
			detailsText color: aColor]
		ifFalse: [
			passFailText insideColor: aColor.
			detailsText insideColor: aColor]! !

!TestBrowser methodsFor: 'updating' stamp: 'minami 11/7/2004 19:10'!
updatePassFail
	| message |
	message := self hasPassed
				ifTrue: ['Pass']
				ifFalse: ['Fail'].
	self displayPassFail: message! !

!TestBrowser methodsFor: 'updating' stamp: 'minami 11/7/2004 19:13'!
updateWindow
	((self errors isEmpty) and: [self failures isEmpty])
		ifTrue: [self updatePartColors: self passColor]
		ifFalse: [self errors isEmpty
					ifTrue: [self updatePartColors: self failColor]
					ifFalse: [self updatePartColors: self errorColor]].

	self updatePassFail.
	self updateDetails.
	self updateFailures.
	self updateErrors.! !

TestCase subclass: #TBAbstractTestCaseMock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TestBrowser-Tests-Utilities'!

!TBAbstractTestCaseMock commentStamp: 'tbn 11/5/2004 13:45' prior: 0!
This is a mock object representing an abstract test case class!

!TBAbstractTestCaseMock class methodsFor: 'testing' stamp: 'minami 11/6/2004 22:35'!
isAbstract

	^self sunitName = #TBAbstractTestCaseMock! !

!TBAbstractTestCaseMock methodsFor: 'testing' stamp: 'tbn 11/5/2004 12:47'!
testFoo
	self assert: true! !

TBAbstractTestCaseMock subclass: #TBConcreteTestCaseMock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TestBrowser-Tests-Utilities'!

!TBConcreteTestCaseMock commentStamp: 'tbn 11/5/2004 13:45' prior: 0!
This is a mock object representing a concrete test case class!

!TBConcreteTestCaseMock methodsFor: 'testing' stamp: 'tbn 11/5/2004 13:27'!
testBar
	
	self assert: true! !

TestCase subclass: #TestBrowserTests
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TestBrowser-Tests'!

!TestBrowserTests commentStamp: 'tbn 11/5/2004 09:47' prior: 0!
This class represents tests for the Squeak TestBrowser!

!TestBrowserTests methodsFor: 'accessing' stamp: 'tbn 11/5/2004 09:46'!
browser
	"Returns the receivers tested browser instance"
	
	^browser! !

!TestBrowserTests methodsFor: 'accessing' stamp: 'tbn 11/5/2004 09:46'!
browser: anObject
	"Sets the receivers tested browser instance"
	
	browser := anObject! !

!TestBrowserTests methodsFor: 'defaults' stamp: 'tbn 11/5/2004 09:13'!
defaultTestClass
	"Returns the receivers default test class"
	
	^TestBrowser! !

!TestBrowserTests methodsFor: 'running' stamp: 'tbn 11/5/2004 09:41'!
setUp
	"Set up the receiver for running a new test"
	
	super setUp.
	self browser: self defaultTestClass new! !

!TestBrowserTests methodsFor: 'running' stamp: 'tbn 11/5/2004 09:42'!
tearDown
	"tear down the receiver after running a test"
	
	super tearDown.
	self browser: nil! !

!TestBrowserTests methodsFor: 'testing' stamp: 'tbn 11/5/2004 09:45'!
testAllTestCaseClasses

	|classes|
	classes := browser allTestCaseClasses.
	self assert: classes notNil.
	self assert: classes isCollection.
	self assert: (classes includes: self class)! !

!TestBrowserTests methodsFor: 'testing' stamp: 'tbn 11/5/2004 09:45'!
testInstanceCreation

	|instance|
	instance := self defaultTestClass new.
	self assert: instance notNil.
	self assert: instance class = self defaultTestClass.
! !

!TestBrowserTests methodsFor: 'testing' stamp: 'minami 7/25/2005 16:21'!
testVersionString

	|version|
	version := self defaultTestClass version.
	self assert: version notNil.
	self assert: (version isKindOf: String)! !

Object subclass: #TBNotATestCaseMock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TestBrowser-Tests-Utilities'!

!TBNotATestCaseMock commentStamp: 'tbn 11/5/2004 13:46' prior: 0!
This is a mock object representing a class that is not a test case!

Object subclass: #TBTestHolder
	instanceVariableNames: 'suite result processingTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TestBrowser'!

!TBTestHolder commentStamp: 'minami 11/8/2004 03:40' prior: 0!
I'm a helper class for TestBrowser. I hold test suites and test results.!

!TBTestHolder methodsFor: 'suite' stamp: 'minami 11/7/2004 16:15'!
buildSuiteClasses: classes 
	suite := self privBuildSuiteClasses: classes ! !

!TBTestHolder methodsFor: 'suite' stamp: 'minami 11/7/2004 15:50'!
buildSuiteClass: classes 
	suite := self privBuildSuiteClasses: (Array with: classes)  ! !

!TBTestHolder methodsFor: 'suite' stamp: 'minami 11/7/2004 16:11'!
buildSuiteClass: clazz methods: selectors
	suite := self privBuildSuiteClass: clazz methods: selectors! !

!TBTestHolder methodsFor: 'suite' stamp: 'minami 11/7/2004 16:15'!
buildSuiteClass: clazz method: selector
	suite := self privBuildSuiteClass: clazz methods: (Array with: selector)! !

!TBTestHolder methodsFor: 'initialize-release' stamp: 'minami 11/7/2004 13:45'!
clear
	suite := nil.
	result := nil.
	processingTime := nil.
	! !

!TBTestHolder methodsFor: 'accessing' stamp: 'minami 11/7/2004 17:42'!
errors
	^ self result errors asSortedCollection: [:a :b | a printString < b printString]! !

!TBTestHolder methodsFor: 'accessing' stamp: 'minami 11/7/2004 17:42'!
failures
	^ self result failures asSortedCollection: [:a :b | a printString < b printString]! !

!TBTestHolder methodsFor: 'testing' stamp: 'minami 11/7/2004 15:15'!
isEmptyResult
	^ self result runCount == 0! !

!TBTestHolder methodsFor: 'testing' stamp: 'minami 11/7/2004 15:15'!
isEmptySuite
	^ self suite tests isEmpty! !

!TBTestHolder methodsFor: 'testing' stamp: 'minami 11/6/2004 22:40'!
isTestCaseClass: aClass 
	"Returns true if the class is a valid test case class"

	^ Preferences testBrowserRunsAbstractClasses
		ifTrue: [aClass inheritsFrom: TestCase]
		ifFalse: [(aClass inheritsFrom: TestCase) and: [aClass isAbstract not]]! !

!TBTestHolder methodsFor: 'suite' stamp: 'minami 11/7/2004 16:16'!
privBuildSuiteClasses: classes 

	| testClasses testSuite |

	testClasses := classes select: [:each | self isTestCaseClass: each].
	testClasses isEmpty ifTrue: [^ TestSuite new].

	testSuite := TestSuite new.
	testClasses
		do: [:testClass | testSuite addTest: testClass suite].
	^ testSuite! !

!TBTestHolder methodsFor: 'suite' stamp: 'minami 11/7/2004 16:12'!
privBuildSuiteClass: clazz methods: selectors 

	| testClass testSelectors |

	(self isTestCaseClass: clazz) ifFalse: [^ TestSuite new].
	testClass := clazz.

	testSelectors := selectors select: [:each | 'test*' match: each].
	testSelectors isEmpty ifTrue: [^ TestSuite new].

	^ testSelectors
		inject: TestSuite new
		into: [:testSuite :testSelector | 
			testSuite
				addTest: (testClass selector: testSelector);
				 yourself]! !

!TBTestHolder methodsFor: 'accessing' stamp: 'minami 11/7/2004 13:44'!
processingTime
	processingTime isNil ifTrue: [processingTime := 0].
	^ processingTime ! !

!TBTestHolder methodsFor: 'printing' stamp: 'minami 11/7/2004 15:16'!
processingTimeString
	| time |
	time := (self isEmptyResult)
				ifTrue: [0]
				ifFalse: [self processingTime = 0
						ifTrue: [1]
						ifFalse: [self processingTime]].
	^ (time / 1000) asFloat printString , ' sec'! !

!TBTestHolder methodsFor: 'accessing' stamp: 'minami 11/7/2004 13:43'!
result
	result isNil ifTrue: [result := TestResult new].
	^ result! !

!TBTestHolder methodsFor: 'printing' stamp: 'minami 11/7/2004 15:16'!
resultString
	^ (self isEmptyResult)
		ifTrue: ['...']
		ifFalse: [self result printString , ', ' , self processingTimeString]! !

!TBTestHolder methodsFor: 'processing' stamp: 'minami 11/7/2004 14:00'!
runCategoryTest: testClasses
	self buildSuiteClasses: testClasses.
	self runSuite! !

!TBTestHolder methodsFor: 'processing' stamp: 'minami 11/7/2004 13:58'!
runClassTest: testClass
	self buildSuiteClass: testClass.
	self runSuite! !

!TBTestHolder methodsFor: 'processing' stamp: 'minami 11/7/2004 14:01'!
runMethodTest: testClass method: testSelector
	self buildSuiteClass: testClass method: testSelector
	self runSuite! !

!TBTestHolder methodsFor: 'processing' stamp: 'minami 11/7/2004 13:45'!
runSuite
	processingTime := Time millisecondsToRun: [result := self suite run].
! !

!TBTestHolder methodsFor: 'accessing' stamp: 'minami 11/7/2004 13:44'!
suite
	suite isNil ifTrue: [suite := TestSuite new].
	^ suite! !
TestBrowser initialize!
